/*
 * packetmodeller/netdevice.h
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __NETDEV_H
#define __NETDEV_H

/**
 *  Functions handling the network interfaces
 */

/** INCLUDES ******************************************************************/
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/ioctl.h>
#include <linux/if.h>
#include <linux/if_tun.h>
#include <linux/if_ether.h>


/** NETWORK DEVICE RELATED PUBLIC FUNCTIONS ***********************************/

/**
 * Return the MAC address of the given interface
 * @param 6 bytes char array to fill in mac address
 * @param device of requested address
 */
int
get_mac_by_dev(unsigned char *, const char *);

/**
 * Return the IP address to a given interface
 * @param struct to fill with ip address
 * @param device of requested address
 */
int
get_ip_by_dev(struct in_addr *, const char *);

/**
 * Create UDP socket and bind to given interface
 * @param device name (e.g. eth0)
 * @param port to bind to
 * @return socket device file descriptor for network device
 */
int
alloc_net_socket(const char *, const uint16_t);

/**
 * Return the character device file descriptor for the TAP device
 */
int
alloc_tap_socket(const char *);

/**
 * Check if it's available device name
 */
int
validate_dev_name(const char *);

/**
 * Read a port from char array, check for validity and return it in
 * network byte order
 * @param port number as characters from stdin
 * @return valid port number
 */
uint16_t
validate_port(const char *);

/**
 * Check IP:Port pair for validity and fill into sockaddr_in struct
 */
int
validate_model_addr(struct sockaddr_in *, const char *);

#endif
