/*
 * packetmodeller/pm_types.h
 *
 * Copyright (C) 2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __PM_TYPES_H
#define __PM_TYPES_H

/**
 * Header to include PacketModeller related types
 */

/** INCLUDES ******************************************************************/
#include <netinet/in.h>
#include <linux/if_ether.h>
#include <linux/types.h>
#include <linux/wireless.h>


/** EXTERNAL IDENTIFIERS (see model.c) ****************************************/
extern int send_freq(int);


/** IW TYPES ******************************************************************/
#ifndef IWLIB_H

/** If we didn't include iwlib.h before, define iw data types ourselves.
 *  This is used to compile WlanModel without dependency to wireless-tools headers
 *  (taken from /usr/include/iwlib.h) */

/** Some usefull constants */
#define KILO    1e3
#define MEGA    1e6
#define GIGA    1e9

/** Shortcuts */
typedef struct iw_statistics    iwstats;
typedef struct iw_range         iwrange;
typedef struct iw_param         iwparam;
typedef struct iw_freq          iwfreq;
typedef struct iw_quality       iwqual;
typedef struct iw_priv_args     iwprivargs;
typedef struct sockaddr         sockaddr;

/** Structure for storing all wireless information for each device */
typedef struct wireless_config
{
  char      name[IFNAMSIZ + 1];             // Wireless/protocol name
  int       has_nwid;
  iwparam   nwid;                           // Network ID
  int       has_freq;
  double    freq;                           // Frequency/channel
  int       freq_flags;
  int       has_key;
  unsigned char key[IW_ENCODING_TOKEN_MAX]; // Encoding key used
  int       key_size;                       // Number of bytes
  int       key_flags;                      // Various flags
  int       has_essid;
  int       essid_on;
  char      essid[IW_ESSID_MAX_SIZE + 1];   // ESSID (extended network)
  int       has_mode;
  int       mode;                           // Operation mode
} wireless_config;

/** Structure for storing all wireless information for each device */
typedef struct wireless_info
{
  struct wireless_config    b;              // Basic information

  int       has_sens;
  iwparam   sens;                           // Sensitivity
  int       has_nickname;
  char      nickname[IW_ESSID_MAX_SIZE + 1]; // NickName
  int       has_ap_addr;
  sockaddr  ap_addr;                        // Access point address
  int       has_bitrate;
  iwparam   bitrate;                        // Bit rate in bps
  int       has_rts;
  iwparam   rts;                            // RTS threshold in bytes
  int       has_frag;
  iwparam   frag;                           // Fragmentation threshold in bytes
  int       has_power;
  iwparam   power;                          // Power management parameters
  int       has_txpower;
  iwparam   txpower;                        // Transmit Power in dBm
  int       has_retry;
  iwparam   retry;                          // Retry limit or lifetime

  /** Stats */
  iwstats   stats;
  int       has_stats;
  iwrange   range;
  int       has_range;

  /** Auth params for WPA/802.1x/802.11i */
  int       auth_key_mgmt;
  int       has_auth_key_mgmt;
  int       auth_cipher_pairwise;
  int       has_auth_cipher_pairwise;
  int       auth_cipher_group;
  int       has_auth_cipher_group;
} wireless_info;

#endif

/** PM VIF DATA TYPES *********************************************************/

typedef unsigned char mac_addr [ETH_ALEN];
typedef char host_id [2 * ETH_ALEN + 1];

/** Shared Memory format */
typedef struct vifd_vif_info {
    char                    vif_name[IFNAMSIZ + 1];
    struct wireless_info    vif_config;
} vifd_vif_info;

/** Full PacketModeller vif configuration */
typedef struct pm_vif_info {
    host_id                 vif_id;
    unsigned char           vif_mac[ETH_ALEN];
    vifd_vif_info *         vif_data;
} pm_vif_info;

typedef struct model_vif_info {
    char                    vif_name[IFNAMSIZ + 1];
    host_id                 vif_id;
    unsigned char           vif_mac[ETH_ALEN];
    double                  freq;
} model_vif_info;

/** Vif parameter format for config_msg */
union vif_param {
    double                  freq;
};

#endif
