/*
 * vifd/vifd_ipc.h
 *
 * Copyright (C) 2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __VIFD_IPC_H
#define __VIFD_IPC_H

/**
 * Defines functions and data types for IPC communication via System V
 * message queues and shared memory
 */

/** INCLUDES ******************************************************************/
#include <errno.h>
#include <iwlib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>


/** IPC VALUES ****************************************************************/

/** define query error (compatible to iwconfig.c) */
#define IWERR_GET_EXT       -7

/** IPC shared memory/message queue key for communication with wireless tools */
#define IW_IPC_KEY          2009

/** IPC shared memory/message queue key for communication with PacketModeller */
#define PM_IPC_KEY          1337

/** Message types
 * -> don't use 0 for message type! (0 = first message in queue) */
#define IW_IOCTL            1   // ioctl request via iwreq
#define IW_ACK              2   // response to ioctl request
#define PM_UPDATE           3   // update request for packetmodeller
#define PM_ACK              4   // response to update request

/** Iwreq acknowledge */
#define IPC_OK              0x0000   // found valid iwreq data (default)
#define IPC_FAIL            0x0001   // error querying iwreq property/device
#define IWREQ_DATA          0x0010   // iwreq response contains shared memory data


/** IPC DATATYPES *************************************************************/

/** Message structure */
struct iwreq_msg {
    long mtype;
    int iwreq_ack;
    int iwreq_property; // property to set/query
    struct iwreq iw_request;
};

struct pm_msg {
    long mtype;
    int vif_index;
    int property;
};

/** Shared Memory format */
struct vif_info {
    char vif_name[IFNAMSIZ + 1];
    struct wireless_info vif_config;
};

/** Message sizes */
#define IW_MSG_SIZE         sizeof(struct iwreq_msg)
#define PM_MSG_SIZE         sizeof(struct pm_msg)


/** PUBLIC FUNCTIONS **********************************************************/

/**
 * Initialize message queue inteface
 */
int
msgq_initialize(int);

/**
 * Receive message from message queue
 */
int
msgq_receive(int, void *, int, int);

/**
 * Delete given message queue
 */
int
msgq_delete(int);

/**
 * Initialize shared memory space of given size
 */
int
shmem_initialize(int, int);

/**
 * Attaches a shared memory segment
 */
char *
shmem_attach(int, char *);

/**
 * Detach a shared memory segment
 */
int
shmem_detach(char *);

/**
 * Free given shared memory space
 */
int
shmem_delete(int);

#endif
